#include "color_scale.h"

typedef uint8_t T_rgb_color;

static const uint8_t RANGE_MIN_LOW = 0;
static const uint8_t RANGE_LOW_MED = 1;
static const uint8_t RANGE_MED_HIG = 2;
static const uint8_t RANGE_HIG_MAX = 3;

const uint8_t B_COLOR = 0;
const uint8_t G_COLOR = 1;
const uint8_t R_COLOR = 2;

static float _abs_min;
static float _abs_max;
static float _low_center;
static float _med_center;
static float _hig_center;

static T_rgb_color _low_center_color;
static T_rgb_color _med_center_color;
static T_rgb_color _hig_center_color;

/* --------------------------------------------------------------- PRIVATE FUNCTIONS DECLARATIONS */

static uint8_t _get_range( float val );

static uint16_t _bw_intensity( float input, T_rgb_color color );

static uint16_t _rgb_intensity( float input, T_rgb_color color );

/* ----------------------------------------------------------------- PUBLIC FUNCTIONS DEFINITIONS */

void color_scale_init()
{
    _abs_min    = (+  18.0);
    _low_center = (+  24.0);
    _med_center = (+  30.0);
    _hig_center = (+  36.0);
    _abs_max    = (+  44.0);
    
    _low_center_color = B_COLOR;
    _med_center_color = G_COLOR;
    _hig_center_color = R_COLOR;
}

int color_scale_set_low_center( float low_center, T_rgb_color color )
{
    _low_center = low_center;
    _low_center_color = color;

    return 0;
}

int color_scale_set_med_center( float med_center, T_rgb_color color )
{
    _med_center = med_center;
    _med_center_color = color;

    return 0;
}

int color_scale_set_hig_center( float hig_center, T_rgb_color color )
{
    _hig_center = hig_center;
    _hig_center_color = color;
    
    return 0;
}

uint16_t color_scale_rgb_conversion( float x )
{  
    uint8_t         range;
    uint16_t        col;

    range = _get_range(x);

    switch (range)
    {
        case RANGE_MIN_LOW:
        {
              float b = (_low_center - x) / (_low_center - _abs_min);
              float c = (x - _abs_min) / (_low_center - _abs_min);
              
              col = _bw_intensity( 1.0 - b, _low_center_color );
              col |= _rgb_intensity( c, _low_center_color );
        break;
        }
        case RANGE_LOW_MED:
        {
              float c1 = (_med_center - x) / (_med_center - _low_center);
              float c2 = (x - _low_center) / (_med_center - _low_center);
              
              col = _rgb_intensity( c1, _low_center_color);
              col |= _rgb_intensity( c2, _med_center_color);
        break;
        }
        case RANGE_MED_HIG:
        {
              float c1 = (_hig_center - x) / (_hig_center - _med_center);
              float c2 = (x - _med_center) / (_hig_center - _med_center);
              
              col = _rgb_intensity( c1, _med_center_color);
              col |= _rgb_intensity( c2, _hig_center_color);

        break;
        }
        case RANGE_HIG_MAX:
        {
              float c = (_abs_max - x) / (_abs_max - _hig_center );
              float w = (x - _abs_max) / (_abs_max - _hig_center );
              
              col = _rgb_intensity( c, _hig_center_color );
              col |= _bw_intensity( w, _hig_center_color );
        break;
        }
    }

    return col;
}

/* ----------------------------------------------------------------- PUBLIC FUNCTIONS DEFINITIONS */

static uint8_t _get_range( float val )
{
    if (val < _low_center)
        return RANGE_MIN_LOW;
    if (val < _med_center)
        return RANGE_LOW_MED;
    if (val < _hig_center)
        return RANGE_MED_HIG;
    return RANGE_HIG_MAX;
}

static uint16_t _bw_intensity( float input, T_rgb_color color )
{
    uint16_t res;
    
    res = 0;

    switch (color)
    {
        case R_COLOR:
            res |= _rgb_intensity( input, G_COLOR );
            res |= _rgb_intensity( input, B_COLOR );
        break;
        case G_COLOR:
            res |= _rgb_intensity( input, R_COLOR );
            res |= _rgb_intensity( input, B_COLOR );
        break;
        case B_COLOR :
            res |= _rgb_intensity( input, G_COLOR );
            res |= _rgb_intensity( input, R_COLOR );
        break;
        default:

        break;
    }

    return res;
}

static uint16_t _rgb_intensity( float input, T_rgb_color color )
{
    uint16_t res;

    switch (color)
    {
        case R_COLOR:
            res = (uint16_t)(input * 32.0);
            res &= 0x1F;
            res <<= 11;
        break;
        case G_COLOR:
            res = (uint16_t)(input * 64.0);
            res &= 0x3F;
            res <<= 5;
        break;
        case B_COLOR :
            res = (uint16_t)(input * 32.0);
            res &= 0x1F;
        break;
        default:

        break;
    }

    return res;
}